//------------------------------------------------------------------------------
// Test Contouring Algorithm for tracked polygons
//  - II/2002 TriplexWare; Written by A.Weidauer
//------------------------------------------------------------------------------
{
@abstract(Unit to realize a polygon based contouring routine.)
@author  (Alexander Weidauer alex.weidauer@huckfinn.de)
@created (February 2002)
@lastmod (February 2002)
The following contouring routine was published in germany 1987 by
Markus Weber Turbopascal Tools Practical usage of Turbo Pascal in
nature science and I adopt it in 1990 for general use from
the third edition of these book ISBN-3-528-24543-3.
}
program TestWeberContur;
{$APPTYPE CONSOLE}
uses
  Windows,
  Graphics,
  sysutils,
  UConturWeber in 'UConturWeber.pas';
//------------------------------------------------------------------------------
Const cHeight=0.3;
      cDebug = True;
      cBMPOverSample = 10;
//------------------------------------------------------------------------------
Var TF:TDataMatrix;
    max,i,j:Integer;
    Contour:TContour;

//------------------------------------------------------------------------------
Procedure SmallHole;
 Begin
  max:=5;
  SetLength(TF,max);
  Writeln('==================================================================');
  Writeln;
  Writeln('----- SmallHole -----------------------------');
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=0;TF[1,0]:=0;TF[2,0]:=0;TF[3,0]:=0;TF[4,0]:=0;
  TF[0,1]:=0;TF[1,1]:=0;TF[2,1]:=1;TF[3,1]:=0;TF[4,1]:=0;
  TF[0,2]:=0;TF[1,2]:=1;TF[2,2]:=0;TF[3,2]:=1;TF[4,2]:=0;
  TF[0,3]:=0;TF[1,3]:=0;TF[2,3]:=1;TF[3,3]:=0;TF[4,3]:=0;
  TF[0,4]:=0;TF[1,4]:=0;TF[2,4]:=0;TF[3,4]:=0;TF[4,4]:=0;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure LargeHole;
 Begin
  max:=5;
  SetLength(TF,max);
  Writeln('==================================================================');
  Writeln;
  Writeln('----- Largehole -----------------------------');
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=0;TF[1,0]:=0;TF[2,0]:=1;TF[3,0]:=0;TF[4,0]:=0;
  TF[0,1]:=0;TF[1,1]:=1;TF[2,1]:=0;TF[3,1]:=1;TF[4,1]:=0;
  TF[0,2]:=1;TF[1,2]:=0;TF[2,2]:=0;TF[3,2]:=0;TF[4,2]:=1;
  TF[0,3]:=0;TF[1,3]:=1;TF[2,3]:=0;TF[3,3]:=1;TF[4,3]:=0;
  TF[0,4]:=0;TF[1,4]:=0;TF[2,4]:=1;TF[3,4]:=0;TF[4,4]:=0;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure RandHole;
 Begin
  Writeln('==================================================================');
  Writeln;
  Writeln('----- Borderhole -----------------------------');
  max:=5;
  SetLength(TF,max);
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=1;TF[1,0]:=0;TF[2,0]:=1;TF[3,0]:=0;TF[4,0]:=0;
  TF[0,1]:=0;TF[1,1]:=1;TF[2,1]:=0;TF[3,1]:=1;TF[4,1]:=0;
  TF[0,2]:=0;TF[1,2]:=0;TF[2,2]:=0;TF[3,2]:=0;TF[4,2]:=0;
  TF[0,3]:=0;TF[1,3]:=0;TF[2,3]:=0;TF[3,3]:=1;TF[4,3]:=0;
  TF[0,4]:=0;TF[1,4]:=0;TF[2,4]:=1;TF[3,4]:=0;TF[4,4]:=1;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure Singles;
 Begin
  Writeln('==================================================================');
  Writeln;
  Writeln('----- Singles -----------------------------');
  max:=5;
  SetLength(TF,max);
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=1;TF[1,0]:=0;TF[2,0]:=1;TF[3,0]:=0;TF[4,0]:=1;
  TF[0,1]:=0;TF[1,1]:=0;TF[2,1]:=0;TF[3,1]:=0;TF[4,1]:=0;
  TF[0,2]:=1;TF[1,2]:=0;TF[2,2]:=1;TF[3,2]:=0;TF[4,2]:=1;
  TF[0,3]:=0;TF[1,3]:=0;TF[2,3]:=0;TF[3,3]:=0;TF[4,3]:=0;
  TF[0,4]:=1;TF[1,4]:=0;TF[2,4]:=1;TF[3,4]:=0;TF[4,4]:=1;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure Chess;
 Begin
  Writeln('==================================================================');
  Writeln;
  Writeln('----- Chess -----------------------------');
  max:=5;
  SetLength(TF,max);
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=1;TF[1,0]:=0;TF[2,0]:=1;TF[3,0]:=0;TF[4,0]:=1;
  TF[0,1]:=0;TF[1,1]:=1;TF[2,1]:=0;TF[3,1]:=1;TF[4,1]:=0;
  TF[0,2]:=1;TF[1,2]:=0;TF[2,2]:=1;TF[3,2]:=0;TF[4,2]:=1;
  TF[0,3]:=0;TF[1,3]:=1;TF[2,3]:=0;TF[3,3]:=1;TF[4,3]:=0;
  TF[0,4]:=1;TF[1,4]:=0;TF[2,4]:=1;TF[3,4]:=0;TF[4,4]:=1;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure Cross1;
 Begin
  Writeln('==================================================================');
  Writeln;
  Writeln('----- Cross1 -----------------------------');
  max:=5;
  SetLength(TF,max);
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=1;TF[1,0]:=0;TF[2,0]:=1;TF[3,0]:=0;TF[4,0]:=1;
  TF[0,1]:=0;TF[1,1]:=1;TF[2,1]:=0;TF[3,1]:=1;TF[4,1]:=0;
  TF[0,2]:=0;TF[1,2]:=0;TF[2,2]:=1;TF[3,2]:=0;TF[4,2]:=0;
  TF[0,3]:=0;TF[1,3]:=1;TF[2,3]:=0;TF[3,3]:=1;TF[4,3]:=0;
  TF[0,4]:=1;TF[1,4]:=0;TF[2,4]:=1;TF[3,4]:=0;TF[4,4]:=1;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure Cross2;
 Begin
  Writeln('==================================================================');
  Writeln;
  Writeln('----- Cross2 -----------------------------');
  max:=5;
  SetLength(TF,max);
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=0;TF[1,0]:=0;TF[2,0]:=1;TF[3,0]:=0;TF[4,0]:=0;
  TF[0,1]:=0;TF[1,1]:=0;TF[2,1]:=1;TF[3,1]:=0;TF[4,1]:=0;
  TF[0,2]:=1;TF[1,2]:=1;TF[2,2]:=1;TF[3,2]:=1;TF[4,2]:=1;
  TF[0,3]:=0;TF[1,3]:=0;TF[2,3]:=1;TF[3,3]:=0;TF[4,3]:=0;
  TF[0,4]:=0;TF[1,4]:=0;TF[2,4]:=1;TF[3,4]:=0;TF[4,4]:=0;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure CrossHole;
 Begin
  Writeln('==================================================================');
  Writeln;
  Writeln('----- CrossHole -----------------------------');
  max:=5;
  SetLength(TF,max);
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=0;TF[1,0]:=1;TF[2,0]:=1;TF[3,0]:=1;TF[4,0]:=0;
  TF[0,1]:=1;TF[1,1]:=1;TF[2,1]:=0;TF[3,1]:=1;TF[4,1]:=1;
  TF[0,2]:=1;TF[1,2]:=0;TF[2,2]:=0;TF[3,2]:=0;TF[4,2]:=1;
  TF[0,3]:=1;TF[1,3]:=1;TF[2,3]:=0;TF[3,3]:=1;TF[4,3]:=1;
  TF[0,4]:=0;TF[1,4]:=1;TF[2,4]:=1;TF[3,4]:=1;TF[4,4]:=0;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure RectBorder;
 Begin
  Writeln('==================================================================');
  Writeln;
  Writeln('----- Rectangle border and inner dot -----------------------------');
  max:=5;
  SetLength(TF,max);
  For i:=0 to max-1 Do Setlength(TF[i],Max);
  TF[0,0]:=1;TF[1,0]:=1;TF[2,0]:=1;TF[3,0]:=1;TF[4,0]:=1;
  TF[0,1]:=1;TF[1,1]:=0;TF[2,1]:=0;TF[3,1]:=0;TF[4,1]:=1;
  TF[0,2]:=1;TF[1,2]:=0;TF[2,2]:=1;TF[3,2]:=0;TF[4,2]:=1;
  TF[0,3]:=1;TF[1,3]:=0;TF[2,3]:=0;TF[3,3]:=0;TF[4,3]:=1;
  TF[0,4]:=1;TF[1,4]:=1;TF[2,4]:=1;TF[3,4]:=1;TF[4,4]:=1;
  RawContour(cDebug,cHeight,TF,Contour);
  Contour.Free;
  Contour:=NIL;
  Write('..next field <enter>'); Readln;
 End;
//------------------------------------------------------------------------------
Procedure MakeBitmap(FileName:String);
Var aBMP:TBitmap;
    x,y:Double;

 Procedure Draw(Color:TColor);
  Var BI,Bj:Integer;
  Begin
   aBMP.Canvas.Pen.Color:=Color;
   aBMP.Canvas.Brush.Style:=bsSolid;
   aBMP.Canvas.Font.Color:=Color;
   With Contour Do Begin
   For bi:=1 To NumOfRings Do Begin
    For bj:= 1 To Rings[bi].NumOfMoves Do
     If bj=1 Then
      aBMP.Canvas.Moveto(
             Round(Rings[bi].Moves[bj].CorrectX(height)*cBMPOverSample),
             Round(Rings[bi].Moves[bj].CorrectY(height)*cBMPOverSample))
     Else
      aBMP.Canvas.LineTo(
             Round(Rings[bi].Moves[bj].CorrectX(height)*cBMPOverSample),
             Round(Rings[bi].Moves[bj].CorrectY(height)*cBMPOverSample))
    End;
     For bi:=1 To NumOfRings Do Begin
     aBMP.Canvas.TextOut(
             Round(Rings[bi].Moves[1].CorrectX(height)*cBMPOverSample),
             Round(Rings[bi].Moves[1].CorrectY(height)*cBMPOverSample),
             Format('%F',[Height]));
     aBMP.Canvas.Ellipse(
             Round(Rings[bi].Moves[1].CorrectX(height)*cBMPOverSample-2),
             Round(Rings[bi].Moves[1].CorrectY(height)*cBMPOverSample-2),
             Round(Rings[bi].Moves[1].CorrectX(height)*cBMPOverSample+2),
             Round(Rings[bi].Moves[1].CorrectY(height)*cBMPOverSample+2));
     End;
    End;
   aBMP.Canvas.Font.Color:=clBlack;
  End;



Begin
 Writeln('==================================================================');
 Writeln;
 Writeln('----- Create function 5 * exp(-sqr(x))*exp(-sqr(y))-2.5 -----------------------------');
 Max:=51;
 SetLength(TF,Max);
 aBMP:=TBitmap.Create;
 aBMP.Width := Max*cBMPOverSample;
 aBMP.Height:= Max*cBMPOverSample;
 aBMP.Pixelformat:=pf24bit;
 For i:=0 to max-1 Do Begin
  Setlength(TF[i],Max);
  x:=(i - (Max-1)/2) / Max * 4;
  For j:=0 To Max-1 Do Begin
   y:=(j - (Max-1)/2) / Max * 4;
   TF[i,j]:=5 * exp(-sqr(x))*exp(-sqr(y))-2.5;
  End;
 End;
 RawContour(False,-2.49,TF,Contour);
 Draw(RGB(0,0, 50));Contour.Clear;
 RawContour(False,-2.25,TF,Contour);
 Draw(RGB(0,0,100));Contour.Clear;
 RawContour(False,-2,TF,Contour);
 Draw(RGB(0,0,100));Contour.Clear;
 RawContour(False,-1.75,TF,Contour);
 Draw(RGB(0,0,150));Contour.Clear;
 RawContour(False,-1.5,TF,Contour);
 Draw(RGB(0,50,150));Contour.Clear;
 RawContour(False,-1.25,TF,Contour);
 Draw(RGB(0,100,150));Contour.Clear;
 RawContour(False,-1.00,TF,Contour);
 Draw(RGB(0,150,150));Contour.Clear;
 RawContour(False,-0.75,TF,Contour);
 Draw(RGB(0,150, 50));Contour.Clear;
 RawContour(False,-0.5,TF,Contour);
 Draw(RGB(0,150, 0));Contour.Clear;
 RawContour(False,-0.25,TF,Contour);
 Draw(RGB(50,150, 0));Contour.Clear;
 RawContour(False,0,TF,Contour);
 Draw(RGB(100,150, 0));Contour.Clear;
 RawContour(False,0.25,TF,Contour);
 Draw(RGB(150,150, 0));Contour.Clear;
 RawContour(False,0.5,TF,Contour);
 Draw(RGB(150,100, 0));Contour.Clear;
 RawContour(False,0.75,TF,Contour);
 Draw(RGB(150, 50, 0));Contour.Clear;
 RawContour(False,1,TF,Contour);
 Draw(RGB(150,  0, 0));Contour.Clear;
 RawContour(False,1.25,TF,Contour);
 Draw(RGB(150,  0, 50));Contour.Clear;
 RawContour(False,1.5,TF,Contour);
 Draw(RGB(150,  0, 100));Contour.Clear;
 RawContour(False,1.75,TF,Contour);
 Draw(RGB(150,  0, 100));Contour.Clear;
 RawContour(False, 2,TF,Contour);
 Draw(RGB(150, 50, 150));Contour.Clear;
 RawContour(False, 2.25,TF,Contour);
 Draw(RGB(150, 100, 150));Contour.Clear;
 RawContour(False, 2.49,TF,Contour);
 Draw(RGB(150, 150, 150));Contour.Clear;
 aBMP.Canvas.TextOut(10,10,'5 * exp(-sqr(x))*exp(-sqr(y))-2.5');
 Writeln('.. storing to file '+FileName);
 aBMP.SaveToFile(FileName);
 Writeln('.. Ready results are in bitmap '+Filename);
 aBMP.Free;
End;


//------------------------------------------------------------------------------
Begin
Writeln('=================================================================');
Writeln(' Test program to create contoures by polygon cursor method of ');
Writeln(' M. Weber, Turbo Pascal Tools, Vieweg 1990, ISBN 3-528-24543-3 ');
Writeln('=================================================================');
Writeln('');
Writeln(' Basic DOS implementation in Turbo PASCAL 4.0 ');
Writeln(' written by (c) - 1990 / M. Weber');
Writeln('');
Writeln('------------------------------------------------------------------');
Writeln('');
Writeln(' Extended implementation with oversampling and border mode  ');
Writeln(' written by (c) - 2001 TriplexWare / A. Weidauer ');
Writeln('');
Writeln('------------------------------------------------------------------');
Writeln(' The algorithm works like a mouse in a labyrinth with a ');
Writeln(' right turning spin if the contact to the wall fails !');
Writeln('------------------------------------------------------------------');
Writeln(' We use a border offset to allow moves along the border to get');
Writeln(' open contours in this case (forbidden side interpolation)!');
Writeln('');
Writeln(' We use oversampling to prevent singularities ');
Writeln(' because in pattern like:');
Writeln('   0 1 2 ');
Writeln('  +----- ');
Writeln(' 0|0 1 0 ');
Writeln(' 1|0 0 0 ');
Writeln(' 2|0 1 0 ');
Writeln(' Contour based on coordinate 1,2 will be overwitten ');
Writeln(' by the original routine of M. Weber');
Writeln('');
Writeln('------------------------------------------------------------------');
Writeln('');
Writeln('Syntax of output table');
Writeln('');
Writeln(' B   - If B cursor is at border no height interpolation available');
Writeln(' MD  - Move direction 0 - East, 1 - South, 2 - West, 3 - North ');
Writeln(' tfx - fload value of the oversapmled x coordinate ');
Writeln(' tfy - fload value of the oversapmled y coordinate ');
Writeln(' tfx - fload value of the oversapmled x coordinate ');
Writeln(' mx  - move x coordinate (stand at the ground of the moutain) ');
Writeln(' my  - move y coordinate (stand at the ground of the moutain) ');
Writeln(' tx  - destination x coordinate (the heigth to climb) ');
Writeln(' ty  - destination y coordinate (the heigth to climb) ');
Writeln(' f(mx,my) - height of the move coordinate ');
Writeln('            NAN if (mx,my) in border');
Writeln(' f(tx,ty) - height of the destination coordinate ');
Writeln('            NAN if (tx,ty) in border');
Writeln('');
Writeln('------------------------------------------------------------------');
Write('..to start press <enter>');Readln;
SmallHole; LargeHole; Randhole;
Singles; Chess; Cross1; Cross2;
CrossHole; RectBorder;
MakeBitmap('test.bmp');
Writeln('==================================================================');
write('..ok thats all, to finish <enter> ');Readln;
Writeln('==================================================================');
end.
