//------------------------------------------------------------------------------
// Core Contouring Algorithm for tracked polygons
//  - II/2002 TriplexWare; Written by A.Weidauer
//------------------------------------------------------------------------------
{
@abstract(Unit to realize a polygon based contouring routine.)
@author  (Alexander Weidauer alex.weidauer@huckfinn.de)
@created (December 1990)
@lastmod (February 2002)
Unit to realize a polygon based contouring routine.

The basic idea of the algorithm is the desciption of the movement of
an unintelligent mouse in a labyrinth with clockwise spin if the move
fails and a counter clock wise spin if the move succeeds for
the next exploring direction.
Versus the contouring other authors like of Paul. D. Brouke
it delivers open or closed polygons (rings) and it is possible to
create faces over on height intersection plane.
To build a DEM or body of heigth lines the algorithm
will take more than height intersections and therefore
more computational power.

The contouring routine was published in germany 1987 by
Markus Weber Turbopascal Tools Practical usage of Turbo Pascal in
nature science and I adopt it in 1990 for general use from
the third edition of these book ISBN-3-528-24543-3.

In technical terms of the time the routine was
implemented as a direct drawing routine to the,
in recent considerations, low resoluted PC DOS screen.
I extract the these parts from the routine and try to
make it industial hard, to check border conditions, uniqe
existence polygons of each height itnersected part.
}
Unit UConturWeber;
//------------------------------------------------------------------------------
Interface
//------------------------------------------------------------------------------
Uses Forms,SysUtils,Classes;

Const
 { Point to mark the data field as visited }
 cVisited      = 0;
 { Point to mark the data field as data point greater than
 the intersection height}
 cData         = 1;
 { Point to mark the data field as smaller than the intersection height}
 cSmaller      = 2;
Type
 {Location type (inside,outside) of the cursor moving arround the binary field}
 TMoveLocation = (tmlInner,tmlBorder);
 { to maskerade the data field }
 TMaskMatrix = Array Of Array of Byte;
 { to declare a data field }
 TDataMatrix = Array Of Array of Double;

//------------------------------------------------------------------------------
 { Representation of one move. The Class has more a didactical
   function so many datas are stored twice. }
 TMoveRecord = Class (TObject)
  { Location of then mouse cursor tmlBorder - cursor is aot of the field
   tmlinner -cursor is inside the field }
  Location     : TMoveLocation;
  { Move direction 0 - west , 1 - south , 2 - east, 3 - north }
  MoveDir      : Byte;
  { X Coord.  of the oversampled data field }
  FloatPosX    : Double;
  { Y Coord.  of the oversampled data field }
  FloatPosY    : Double;
  { X Coord.  of mouse position }
  MovePosX     : Integer;
  { Y Coord.  of mouse position }
  MovePosY     : Integer;
  { X Coord. of target cell for height interpolation }
  DestPosX     : Integer;
  { Y Coord. of target cell for height interpolation }
  DestPosY     : Integer;
  { Height of MovePosX,MovePosY ! if Location is tmlBorder the
    height is setted to the intersection height }
  MoveHeight   : Double;
  { Height of DestPosX,DestPosY }
  DestHeight   : Double;
  { .. to assign datas }
  Procedure Assign(aMove:TMoveRecord);
  { create a string from the datas for debugging }
  Function  ToString(IsectHeight:Double):String;
  { X Coord. height corrected position (intersection with the given height). }
  Function  CorrectX(IsectHeight:Double):Double;
  { X Coord. height corrected position (intersection with the given height). }
  Function  CorrectY(IsectHeight:Double):Double;
 End;
 // ----------------------------------------------------------------------------
 { Representation of one contoured object }
 TContourRing  = Class (TObject)
  Private
  { Data container }
   fList:TList;
  Public
   { ... to create }
   Constructor Create;
   { ... to destroy }
   Destructor  Destroy;
   { Add a move record }
   Procedure   Add(aMove:TMoveRecord);
   { Add a .. to get access via properties }
   Procedure   Put(Index:Integer;aMove:TMoveRecord);
   { Add a .. to get access via properties }
   Function    Get(Index:Integer):TMoveRecord;
   { .. to  assign som datas }
   Procedure   Assign(aRing:TContourRing);
   { Number of moves inside a ring }
   Function    NumOf:Integer;
   { Clear all moves from a ring }
   Procedure   Clear;
   { Number of moves inside a ring }
   Property    NumOfMoves:Integer Read NumOf;
   { Access to each move of the mouse cursor }
   Property    Moves[Index:Integer]:TMoveRecord Read Get Write Put;
 End;
 // ----------------------------------------------------------------------------
 { A contour describes a complex set of contouring polygons over ONE
   intersection height }
 TContour = Class (TObject)
  Private
   { Data container }
   fList:TList;
  Public
   { Intersection Height }
   Height:Double;
   { ... to create }
   Constructor Create;
   { ... to destroy }
   Destructor  Destroy;
   { Add a ring to a contour }
   Procedure   Add(aRing:TContourRing);
   { Add a .. to get access via properties }
   Procedure   Put(Index:Integer;aRing:TContourRing);
   { Add a .. to get access via properties }
   Function    Get(Index:Integer):TContourRing;
   { Number of rings in a countour }
   Function    NumOf:Integer;
   { Clear all rings from a contour }
   Procedure   Clear;
   { Write out the structure to stdIO ! use $APPTYPE CONSOLE }
   Procedure   DebugSTDIO;
   { Number of rings in a countour }
   Property    NumOfRings:Integer Read NumOf;
   { Access to each ring  of each mouse cursor }
   Property    Rings[Index:Integer]:TContourRing Read Get Write Put;
 End;

//------------------------------------------------------------------------------
{ Create a contour over one intersection height by a given
  data field .
  Debug - If TRUE .. Write out all infos to STDIO ! use $APPTYPE CONSOLE and
          datafields < 32x32
  Height - Is Intersection height.! The interpolation will in every
  time treated as <=Height.
  Data - your data field.
  Contour the resulting contour. }
Procedure RawContour (Debug:Boolean;
                      Height:Double;
                      Data: TDataMatrix;
                      Var Contour:TContour);
//------------------------------------------------------------------------------
Implementation

//------------------------------------------------------------------------------
Procedure RawContour (Debug:Boolean;
                      Height:Double;
                      Data: TDataMatrix;
                      Var Contour:TContour);
//------------------------------------------------------------------------------
var
    i,x,y,xt,yt,vl : Integer;
    Mask           : TMaskMatrix;
    finished       : boolean;
    MaxX,MaxY      : Integer;
    mMaxX,mMaxY    : Integer;
    Border         : Boolean;
    ivl            : Integer;
    StartMove      : TMoveRecord;
    NextMove       : TMoveRecord;
    StartString    : String;
    ShowStart      : Boolean;


   Function MaskToData(mX,mY:Integer;Var dX,dY:Integer):Boolean;
    Begin
     result:=true;
     dX:=mX div 2 - 1;
     dY:=mY div 2 - 1;
     If (dX<0) Or (dX>=MaxX) or
        (dY<0) Or (dY>=MaxY) Then Result:=False;
    End;

   Procedure MaskToDataFloat(mX,mY:Integer;Var dX,dY:Double);
    Begin
     dX:=mX / 2 - 1;
     dY:=mY / 2 - 1;
    End;

  //-------------- Suchprocedure -----------------------------------------
  Procedure SEARCH (x,y,d:integer);
  var count: Longint; // counter
      xs,ys:Double;   // coordinates of Startpoint
      xo,yo:Double;   // prevoius point
  //---------------------------------------------------------------------------
   { test of Searchdirection }
   Function Border(x,y:Double):Boolean;
    Begin
     Result:=False;
     If (X<0) Or (X>=MaxX) or
        (Y<0) Or (Y>=MaxY) Then Result:=True;
    End;

   Function TEST (d:byte): boolean;
   { Direction test for the turn around pointer d = 0..3
     to cut point of the mask array
     cSmaller  height of point is smaller then the current height
     cVisted   height was registered before }
     var b: integer;
     begin
      case d of
        0: b := Mask[x-1,y];
        1: b := Mask[x,y+1];
        2: b := Mask[x+1,y];
        3: b := Mask[x,y-1];
       end;
       test := (b=cVisited) or (b=cSmaller) ;
    end;

  //---------------- Fine Starpoint ------------------------------------
  procedure MOVESTART;
  { Setzt start point. }
   var tx,ty  : integer;
       tfx,tfy  : Double;
       isBorder : Boolean;
    begin
     MaskToData(x,y,tx,ty);
     MaskToDataFloat(x,y,tfx,tfy);

     StartMove:=TMoveRecord.Create;
     isBorder:=Border(tx,ty);


     If IsBorder Then Begin
      StartMove.Location:=tmlBorder;
      StartMove.MoveHeight:=Height;
     End Else Begin
      StartMove.Location:=tmlInner;
      StartMove.MoveHeight:=Data[tx,ty];
     End;
     StartMove.MoveDir :=1;
     StartMove.FloatPosX:=tfx;
     StartMove.FloatPosY:=tfy;
     StartMove.MovePosX:=tx;
     StartMove.MovePosY:=ty;
     StartMove.DestPosX:=tx;
     StartMove.DestPosY:=ty+1;
     StartMove.DestHeight:=Data[tx,ty+1];

     xo:=x; yo:=y; // store prevouis point
    end;

  // ---------------------- Interpolation ------------------------------------

  procedure INTERPOLATE (d: byte);

  { Determening to each point the height coordinates }
  var sx,sy,
      tx,ty: Integer;
      tfx,tfy:Double;
      ot,dt:Double;
      isBorder:Boolean;
  Begin
     MaskToData(x,y,tx,ty);
     MaskToDataFloat(x,y,tfx,tfy);

     case d of
        0:  Begin sx:=tx-1;sy:=ty;   End;
        1:  Begin sx:=tx  ;sy:=ty+1; End;
        2:  Begin sx:=tx+1;sy:=ty;   End;
        3:  Begin sx:=tx  ;sy:=ty-1; End;
       end;


     NextMove:=TMoveRecord.Create;
     isBorder:=Border(tx,ty);

     If IsBorder Then Begin
      NextMove.Location:=tmlBorder;
      NextMove.MoveHeight:=Height;
     End Else Begin
      NextMove.Location:=tmlInner;
      NextMove.MoveHeight:=Data[tx,ty];
     End;
     NextMove.MoveDir :=d;
     NextMove.FloatPosX:=tfx;
     NextMove.FloatPosY:=tfy;
     NextMove.MovePosX:=tx;
     NextMove.MovePosY:=ty;
     NextMove.DestPosX:=sx;
     NextMove.DestPosY:=sy;
     NextMove.DestHeight:=Data[sx,sy];

     finished := ((xo=x) and (yo=y)) ; { Startpoint is end point }
    If NOT finished Then begin
     If ShowStart Then Begin
      Contour.Add(TContourRing.Create);
      Contour.Rings[Contour.NumOfRings].Add(StartMove);
      ShowStart:=False;
     End;
      Contour.Rings[Contour.NumOfRings].Add(NextMove);
    end;
  end;   { Interpolate }
//--------------------------------------------------------------------------------------
begin
 finished := false;
 showStart:= True;
 movestart;                             { Erster Punkt }
 repeat
  d := (d+1) and 3;                     { Neues d }
  count := 0;
  if (not test(d)) or finished then     { Nchster Punkt }
   repeat
    Interpolate(d);
    inc(count);
    d := (d+3) and 3;
   until test(d) or (count=5) or finished;

  If (count<5) and not finished then
   begin
    case d of
     0: x := x-1;
     1: y := y+1;
     2: x := x+1;
     3: y := y-1;
    end;
    Mask[x,y]:=cVisited;
   End;
 until finished or (count=5);
 // --- only a move rudiment ----------------------------------------
 If Contour.Rings[Contour.NumOfRings].NumOfMoves=1 Then StartMove.Free;
End;
//-----------------------------------------------------------------------------------
Begin
  MaxX:=High(Data)+1;
  If MaxX=0 Then Exit;
  MaxY:=High(Data[1])+1;
  mMaxX:=2*MaxX+4;
  mMaxY:=2*MaxY+4;
  SetLength(Mask,mMaxX);
  For x:=0 To mMaxX-1 Do Begin
   SetLength(Mask[x],mMaxY);
   For y:=0 To MaxY-1 Do
   Mask[x,y]:=cSmaller;
  End;
//------------------------------------------------------------------------------
  For y := 0 to mMaxY-1 do             { Schnittarray ermitteln }
   For x := 0 to mMaxX-1 do Begin
    vl:=cSmaller;
    If MaskToData(x,y,xt,yt) Then
     If Data[xt,yt]>Height Then vl:=cData else vl:=cSmaller;
    Mask[x,y]:=vl;
   End;
   If Debug Then Begin
    Writeln;
    Writeln(' --- original array ---');
    Writeln;
    Write('   ');
    For x := 0 to MaxX-1 do
      Write(IntToStr(x),' ');
    For y := 0 to MaxY-1 do Begin
      Writeln; Write(' ',IntToStr(y));
      For x := 0 to MaxX-1 do Begin
       If Data[x,y]>Height Then
        Write(' X') else Write(' .');
      End;
    End;
    Writeln;
    Writeln;
    Writeln(' --- over sampled array with border ---');
    Writeln;
    Write('   ');
    For x := 1 to mMaxX-1 do
      Write(IntToStr(x div 2 - 1 ));
    For y := 0 to mMaxY-1 do Begin
      Writeln;Write(Format('%2d ',[y div 2 - 1]));
      For x := 0 to mMaxX-1 do
       If Mask[x,y]=cData Then
        Write('X') else Write('.');
    End;
    Writeln;
    Writeln;
  End;

  If Contour = NIL Then Contour:=TContour.Create;
  Contour.Height:=Height;
  For x := 0 to mMaxX-1 do
   For y := 0 to mMaxY-1 do
   If (Mask[x,y]=cData) AND (Mask[x,y-1]=cSmaller) Then Begin
    search(x,y-1,3);
   End;

  If Debug Then
   Contour.DebugSTDIO;
end; { of Contour}

// ----------------------------------------------------------------------------
// TMoveRecord
// ----------------------------------------------------------------------------
Function  TMoveRecord.CorrectX(IsectHeight:Double):Double;
 Begin
  Result:=MovePosX;
  Case MoveDir of
        0: Result := MovePosX-(IsectHeight-MoveHeight)/(DestHeight-MoveHeight);
        2: Result := MovePosX+(IsectHeight-MoveHeight)/(DestHeight-MoveHeight);
   End;
 End;
// ----------------------------------------------------------------------------
Function  TMoveRecord.CorrectY(IsectHeight:Double):Double;
 Begin
  Result:=MovePosY;
  Case MoveDir of
        1: Result := MovePosY+(IsectHeight-MoveHeight)/(DestHeight-MoveHeight);
        3: Result := MovePosY-(IsectHeight-MoveHeight)/(DestHeight-MoveHeight);
       End
 End;
// ----------------------------------------------------------------------------
Procedure TMoveRecord.Assign(aMove:TMoveRecord);
 Begin
  Location     := aMove.Location;
  MoveDir      := aMove.MoveDir;
  FloatPosX    := aMove.FloatPosX;
  FloatPosY    := aMove.FloatPosY;
  MovePosX     := aMove.MovePosX;
  MovePosY     := aMove.MovePosY;
  DestPosX     := aMove.DestPosX;
  DestPosY     := aMove.DestPosY;
  MoveHeight   := aMove.MoveHeight;
  DestHeight   := aMove.DestHeight;
 End;

// ----------------------------------------------------------------------------
Function TMoveRecord.ToString(IsectHeight:Double):String;
Var S:String;
Begin
 If Location=tmlInner Then S:='I' else S:='B';
 If Location = tmlBorder  Then
   Result:=Format('%S | %2d | %f %f | %2d %2d | %2d %2d | NAN %F ',
    [S,MoveDir,
     CorrectX(IsectHeight),CorrectY(IsectHeight),
     MovePosX ,MovePosY,
     DestPosX ,DestPosY,
     DestHeight])
   Else
     Result:=Format('%S | %2d | %f %f | %2d %2d | %2d %2d | %F %F ',
    [S,MoveDir,
     CorrectX(IsectHeight),CorrectY(IsectHeight),
     MovePosX ,MovePosY,
     DestPosX ,DestPosY,
     MoveHeight,DestHeight]);
End;
// ----------------------------------------------------------------------------
//  TContourRing Routines
// ----------------------------------------------------------------------------

Constructor TContourRing.Create;
 Begin
  Inherited Create;
  fList:=TList.Create;
 End;

// ----------------------------------------------------------------------------
Destructor  TContourRing.Destroy;
 Begin
  fList.Clear;
  fList.Free;
  Inherited Destroy;
 End;

// ----------------------------------------------------------------------------
Procedure TContourRing.Add(aMove:TMoveRecord);
Begin
  fList.Add(aMove);
End;

// ----------------------------------------------------------------------------
Procedure TContourRing.Put(Index:Integer;aMove:TMoveRecord);
Begin
  TMoveRecord(fList.Items[Index - 1]).Assign(aMove);
End;

// ----------------------------------------------------------------------------
Function  TContourRing.Get(Index:Integer):TMoveRecord;
 Begin
  Result := Nil; Result := TMoveRecord(flist.Items[Index - 1]);
 End;

// ----------------------------------------------------------------------------
Function  TContourRing.NumOf:Integer;Begin
 Result:=fList.Count; End;

// ----------------------------------------------------------------------------
Procedure   TContourRing.Clear;
Var i: Integer;
Begin
  For i := 1 To NumOf Do Begin
    TMoveRecord(fList.Items[i - 1]).Free;
    fList.Items[i - 1] := Nil;
  End;
  flist.Pack; fList.Clear;
End;

// ----------------------------------------------------------------------------
Procedure TContourRing.Assign(aRing:TContourRing);
Var i, m: Integer;ToAdd:TMoveRecord;
Begin
  Clear;
  m := aRing.NumOf;
  For i := 1 To m Do Begin
    ToAdd:=TMoveRecord.Create;ToAdd.Assign(aRing.Moves[i]);
    Add(ToAdd);
  End;
End;

// ----------------------------------------------------------------------------
//  TContourRing Routines
// ----------------------------------------------------------------------------
Constructor TContour.Create;
 Begin
  Inherited Create;
  fList:=TList.Create;
 End;

// ----------------------------------------------------------------------------
Destructor  TContour.Destroy;
 Begin
  fList.Clear;
  fList.Free;
  Inherited Destroy;
 End;

// ----------------------------------------------------------------------------
Procedure   TContour.Add(aRing:TContourRing);
Begin
  fList.Add(aRing);
End;

//------------------------------------------------------------------------------
Procedure   TContour.Put(Index:Integer;aRing:TContourRing);
Begin
 fList.Items[Index - 1]:=aRing;
End;

//------------------------------------------------------------------------------
Function    TContour.Get(Index:Integer):TContourRing; Begin
 Result:=NIL;
 Result:=TContourRing(fList.Items[Index - 1]);
End;

Function    TContour.NumOf:Integer; Begin
 Result:=fList.Count;
End;

//------------------------------------------------------------------------------
Procedure   TContour.Clear;
Var i: Integer;
Begin
  For i := 1 To NumOf Do Begin
    TContourRing(fList.Items[i - 1]).Free;
    fList.Items[i - 1] := Nil;
  End;
  flist.Pack; fList.Clear;
End;

//------------------------------------------------------------------------------
Procedure TContour.DebugSTDIO;
 Var i,j:Integer;
 Begin
  For i:=1 To NumOfRings Do Begin
   If i>1 then Begin
    Writeln;
    Write('..show next ring <Enter>'); Readln;
   End;
   Writeln('B | MD | tfx  tfy  | mx my | tx ty | f(mx,my) f(tx,ty) ');
   Writeln('--+----+-----------+-------+-------+-------------------');
   For j:= 1 To Rings[i].NumOfMoves Do
      Writeln(Rings[i].Moves[j].ToString(Height));
  End;
 End;
//------------------------------------------------------------------------------
end.
//------------------------------------------------------------------------------
